"""Configuration models for agent capabilities."""

from llmling_agent.config.capabilities import Capabilities
from llmling_agent.config.knowledge import Knowledge

__all__ = ["Capabilities", "Knowledge"]
